//  Demo 6: SmoothlyMovingSquareApplet.java

//
//     A Simple Java Applet to Use the Square Class
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999
//
// A Square smoothly moving diagonally across the screen.
// Illustrates Graphics double buffering to smooth animation,
// as well as use of the Square class.

// The idea behind double buffering is that we create our own
// Graphics object to use as a graphics buffer.  We draw on
// the buffer in the background, and then simply switch a pointer
// to bring the background buffer onto the screen.  Because
// drawing routines are frequently time consuming, this smooths
// animation -- we don't see the flickering from the drawing 
// process, we just see the finished product.
//
// Unfortunately, as we shall see, this simple idea was rendered
// more complex by the genii at Sun Microsystems.  But in defense
// of Java, all the gory system specific details are hidden, so
// we probably come out ahead in the long run.

// There is one more difficulty.  This is an Applet because 
// calling the createImage method, which we have to do to
// create our own graphics object to draw on, is more difficult
// in applications.  We shall see how to call it in applications
// very shortly.

import java.awt.*;
import Square;
import java.applet.Applet;

public class SmoothlyMovingSquareApplet extends Applet {

   Square s;  // our Square

   Graphics gBuffer;   // Graphics buffer for off-screen drawing
   Image imageBuffer;  // need an Image object to create our
                       // own graphics object, for obscure
                       // Java reasons that only Sun understands

   // init(): as usual, initializes variables, but also
   //         constructs the graphics buffer
   public void init() {

      s = new Square(50, 50, 25, Color.red);  // initialize object
                                              // state variable s

     // create the gBuffer Graphics buffer object
     //   two step process, more Java obscurity
     imageBuffer = createImage(400, 400);  // 400 by 400 pixels
     gBuffer = imageBuffer.getGraphics();

   } // init()

   public void paint(Graphics g) {

     Color backgroundColor = getBackground();

     // Here comes the animation!
     for(int i = 1; i <= 200; i++) {

        // move -- do drawing offscreen on the Graphics
        //         buffer object
        s.draw(gBuffer, backgroundColor);  // hide square
        s.move(1, 1);
        s.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        // To pause, so the animation doesn't go too fast, use
        // the sleep() method of the Thread class.  Unfortunately,
        // it throws an InterruptedException, so we have to embed
        // our simple pause statement in a try -- catch clause.
        try {
          Thread.sleep(20);  // 20 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }
     }
   } // paint

} // class SmoothlyMovingSquareApplet
        
